package com.faraphel.tasks_valider

import android.content.Context
import android.content.pm.PackageManager
import android.Manifest
import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.net.wifi.p2p.*
import android.net.wifi.p2p.WifiP2pManager.ConnectionInfoListener
import android.net.wifi.p2p.WifiP2pManager.GroupInfoListener
import android.os.Build
import android.os.Bundle
import android.util.Log
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.annotation.RequiresApi
import androidx.compose.material3.Button
import com.faraphel.tasks_valider.connectivity.DemoClient
import com.faraphel.tasks_valider.connectivity.DemoServer
import com.faraphel.tasks_valider.connectivity.wifi_p2p.WifiP2pBroadcastReceiver
import com.faraphel.tasks_valider.database.Database
import java.net.InetSocketAddress
import java.net.ServerSocket
import java.net.Socket


class MainActivity : ComponentActivity() {
    private var PERMISSION_ACCESS_FINE_LOCATION = 1001  ///< permission code for the fine location (required for WiFi-Direct)

    private var p2pManager: WifiP2pManager? = null  ///< the wifi direct manager
    private var p2pChannel: WifiP2pManager.Channel? = null  ///< the wifi direct channel
    private var p2pReceiver: BroadcastReceiver? = null  ///< the wifi direct event receiver handler

    companion object {
        private lateinit var database: Database  ///< the database manager
    }

    @RequiresApi(Build.VERSION_CODES.O)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        // ----- CONNECTION

        // TOOD: more check on permissions
        if (!this.packageManager.hasSystemFeature(PackageManager.FEATURE_WIFI_DIRECT)) {
            Log.e("wifi-p2p", "this device does not support the WiFi-Direct feature")
            return
        }

        if (this.checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            // TODO(Faraphel): should be used with shouldShowRequestPermissionRationale, with a check
            this.requestPermissions(arrayOf(Manifest.permission.ACCESS_FINE_LOCATION), PERMISSION_ACCESS_FINE_LOCATION)
        }

        // get the WiFi-Direct manager
        this.p2pManager = this.getSystemService(Context.WIFI_P2P_SERVICE) as WifiP2pManager?
        if (this.p2pManager == null) {
            Log.e("wifi-p2p", "cannot access the WiFi-Direct manager")
            return
        }

        // get the Wifi-Direct channel
        this.p2pChannel = this.p2pManager!!.initialize(this, this.mainLooper, null)
        if (this.p2pManager == null) {
            Log.e("wifi-p2p", "cannot access the WiFi-Direct channel")
            return
        }

        // create a handler for the WiFi-Direct events
        this.p2pReceiver = WifiP2pBroadcastReceiver(this.p2pManager!!, this.p2pChannel!!)

        // start a demo connexion
        this.demoP2pConnexion()
    }

    private fun demoP2pConnexion() {
        // Get the connection information (for the host IP)
        this.p2pManager!!.requestConnectionInfo(this.p2pChannel) { connection ->

            Log.v("wifi-p2p", "connection information: $connection")

            // Get the group information (for the owner)
            this.p2pManager!!.requestGroupInfo(this.p2pChannel) { group ->

                Log.v("wifi-p2p", "group information: $group")

                if (group.isGroupOwner) {
                    Log.d("wifi-p2p", "mode: server")
                    DemoServer(9876).start()
                } else {
                    Log.d("wifi-p2p", "mode: client")

                    Thread {  // NOTE(Faraphel): this thread is started to avoid blocking the main loop

                        while (true) {

                            // TODO(Faraphel): the thread is handled outside the connect block to prevent connection before the server started.
                            // This should be handled in the future by selecting the server before starting the client.
                            p2pManager!!.connect(
                                this.p2pChannel,
                                WifiP2pConfig().apply { deviceAddress = group.owner.deviceAddress },
                                object : WifiP2pManager.ActionListener {
                                    override fun onSuccess() {
                                        Log.d("wifi-p2p", "peer connection successful !")
                                        Thread {
                                            try {
                                                DemoClient(connection.groupOwnerAddress, 9876).once()
                                            } catch(exception: Exception) {
                                                Log.e("wifi-p2p", "could not connect to the server", exception)
                                            }
                                        }.start()  // NOTE(Faraphel): this thread is started to avoid networking in main thread
                                    }

                                    override fun onFailure(reason: Int) {
                                        Log.e("wifi-p2p", "could not establish a peer connection. Reason: $reason")
                                    }
                                }
                            )

                            Thread.sleep(1000)
                        }

                    }.start()
                }
            }
        }
    }

    @RequiresApi(Build.VERSION_CODES.O)
    @SuppressLint("UnspecifiedRegisterReceiverFlag")
    override fun onResume() {
        super.onResume()

        // enable the WiFi-Direct events
        this.registerReceiver(this.p2pReceiver, WifiP2pBroadcastReceiver.INTENT_FILTER)
    }

    override fun onPause() {
        super.onPause()

        // disable the WiFi-Direct events
        this.unregisterReceiver(this.p2pReceiver)
    }
}
